@@ -4,165 +4,25 @@ from collections import defaultdict
 import sys
 import warnings
 
-from .connection import get_database, pymongo_supports_mongoclient
+from bson import ObjectId
+
 from .exceptions import MultipleDocumentsFound, NoDocumentFound
+from .meta import Meta
 from .query import Q, QuerySet
 from .utils import (current_datetime, get_nested_key, guarantee_object_id,
                     is_atomic, map_fields, remove_nested_key,
-                    update_nested_keys)
+                    set_write_concern, update_nested_keys)
 
 __all__ = ('Model',)
 
 
 # In version 2.4, PyMongo introduced MongoClient as a
-# replcaement for Connection. Part of the new class is that the
+# replacement for Connection. Part of the new class is that the
 # safe argument was deprecated in favor of the w argument.
 # Whereas safe was a boolean, w is an integer representing the
 # number of servers in a replica set that must receive the
 # update before the write is considered successful.
 
-def _set_write_concern_as_safe(options, force_safe):
-    """Sets the safe parameter for write concern."""
-
-    safe = options.pop('safe', None)
-    w = options.pop('w', None)
-
-    if force_safe:
-        safe = True
-    else:
-        if safe is None and w:
-            safe = True
-        else:
-            safe = safe or False
-    options['safe'] = safe
-
-
-def _set_write_concern_as_w(options, force_safe):
-    """Sets w safe parameter for write concern."""
-
-    safe = options.pop('safe', None)
-    w = options.pop('w', None)
-
-    if force_safe:
-        w = int(force_safe)  # use int() in case force_safe is True
-    else:
-        if w is None and safe:
-            w = 1
-        else:
-            w = w or 0
-    options['w'] = w
-
-
-if pymongo_supports_mongoclient:
-    _set_write_concern = _set_write_concern_as_w
-else:
-    _set_write_concern = _set_write_concern_as_safe
-
-
-class Meta(object):
-    """Custom options for a :class:`~simon.Model`.
-
-    .. versionadded:: 0.1.0
-    """
-
-    _db = None
-
-    def __init__(self, meta):
-        self.meta = meta
-
-        # Set all the default option values.
-        self.auto_timestamp = True
-        self.database = 'default'
-        self.field_map = {}
-        self.map_id = True
-        self.required_fields = None
-        self.safe = True
-        self.sort = None
-
-    def add_to_original(self, cls, name):
-        """Adds the ``Meta`` object to another class.
-
-        .. versionadded:: 0.1.0
-        """
-
-        cls._meta = self
-
-        if self.meta:
-            meta_attrs = self.meta.__dict__.copy()
-            for name in self.meta.__dict__:
-                # Remove any "private" attributes.
-                if name.startswith('_'):
-                    del meta_attrs[name]
-
-            # Add the known attributes to the instance
-            for name in ('auto_timestamp', 'collection', 'database',
-                         'field_map', 'map_id', 'required_fields', 'safe',
-                         'sort'):
-                if name in meta_attrs:
-                    setattr(self, name, meta_attrs.pop(name))
-
-            if not hasattr(self, 'collection'):
-                # If there's no collection name, generate one from the
-                # other class's name.
-                self.collection = ''.join([cls.__name__.lower(), 's'])
-
-        else:
-            # If there's no real instance to add, we just have to
-            # generate the collection name.
-            self.collection = ''.join([cls.__name__.lower(), 's'])
-        del self.meta
-
-        # Store the name of the class to make __repr__() and __str__()
-        # more useful.
-        self.class_name = cls.__name__
-
-        # Get the list of the attributes associated with the class.
-        # These will make up the list of reserved words that cannot be
-        # used for keys.
-        self.core_attributes = cls.__dict__.keys() + ['_document']
-
-        if self.map_id and 'id' not in self.field_map:
-            # If map_id is True and id isn't in field_map, add it.
-            self.field_map['id'] = '_id'
-
-        # Any of the methods that check for required fields are looking
-        # for something like a list or tuple of fields, not a string
-        # of one field. If a single field name has been provided as a
-        # string, switch it to a tuple now.
-        if self.required_fields and not isinstance(self.required_fields,
-                                                   (list, tuple)):
-            self.required_fields = (self.required_fields,)
-
-        # When calling QuerySet.sort(), it accepts the sort fields as
-        # *args. Forcing Meta.sort into an iterable now allows it to
-        # be specified as either a single field or a list when defining
-        # the model and saves the trouble of having to worry about it
-        # later.
-        if self.sort and not isinstance(self.sort, (list, tuple)):
-            self.sort = (self.sort,)
-
-    @property
-    def db(self):
-        """Contains the :class:`~pymongo.collection.Collection`.
-
-        .. versionadded:: 0.1.0
-        """
-
-        if self._db is None:
-            # Only make the call to get_database once for each class.
-            self._db = get_database(self.database)[self.collection]
-        return self._db
-
-    def __repr__(self):
-        return '<Meta options for {0}>'.format(self.class_name)
-
-    def __str__(self):
-        return '{0}.Meta'.format(self.class_name)
-
-    def __unicode__(self):
-        return u'{0}.Meta'.format(self.class_name)
-
-
 class ModelMetaClass(type):
     """Define :class:`Model`
 
@@ -232,7 +92,7 @@ class Model(object):
         # used to store the real document's values
         self._document = {}
 
-        fields = map_fields(self.__class__, fields)
+        fields = map_fields(self.__class__._meta.field_map, fields)
 
         # Add the fields to the document
         for k, v in fields.items():
@@ -253,7 +113,7 @@ class Model(object):
         return self.find()
 
     @classmethod
-    def create(cls, safe=False, **fields):
+    def create(cls, **fields):
         """Creates a new document and saves it to the database.
 
         This is a convenience method to create a new document. It will
@@ -264,9 +124,11 @@ class Model(object):
         :class:`TypeError` will be raised if any of the fields are not
         provided.
 
-        :param safe: (optional) Whether to perform the create in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :param \*\*fields: Keyword arguments to add to the document.
         :type \*\*fields: \*\*kwargs.
         :returns: :class:`~simon.Model` -- the new document.
@@ -275,12 +137,17 @@ class Model(object):
         .. versionadded:: 0.1.0
         """
 
+        write_concern = {
+            'safe': fields.pop('safe', None),
+            'w': fields.pop('w', None),
+        }
+
         new = cls(**fields)
-        new.save(safe=safe)
+        new.save(**write_concern)
 
         return new
 
-    def delete(self, safe=False):
+    def delete(self, **kwargs):
         """Deletes a single document from the database.
 
         This will delete the document associated with the instance
@@ -288,9 +155,11 @@ class Model(object):
         most likely indicate that the document has never been saved--
         a :class:`TypeError` will be raised.
 
-        :param safe: (optional) Whether to perform the delete in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :raises: :class:`TypeError`
 
         .. versionadded:: 0.1.0
@@ -302,10 +171,14 @@ class Model(object):
                             "'{1}' attribute has not been set.".format(
                                 self.__class__.__name__, '_id'))
 
-        kwargs = {'safe': safe}
-        _set_write_concern(kwargs, self._meta.safe)
+        write_concern = {
+            'safe': kwargs.pop('safe', None),
+            'w': kwargs.pop('w', None),
+        }
 
-        self._meta.db.remove({'_id': id}, **kwargs)
+        set_write_concern(write_concern, self._meta.write_concern)
+
+        self._meta.db.remove({'_id': id}, **write_concern)
 
         self._document = {}
 
@@ -327,7 +200,7 @@ class Model(object):
                   objects matching ``query``.
 
         .. versionchanged:: 0.3.0
-           Deprecating ``qs`` in favor of ``q``
+           ``qs`` is being deprecated in favor of ``q``
 
         .. versionadded:: 0.1.0
         """
@@ -359,7 +232,7 @@ class Model(object):
                  :class:`~simon.Model.NoDocumentFound`
 
         .. versionchanged:: 0.3.0
-           Deprecating ``qs`` in favor of ``q``
+           ``qs`` is being deprecated in favor of ``q``
 
         .. versionadded:: 0.1.0
         """
@@ -373,7 +246,7 @@ class Model(object):
         return cls._find(find_one=True, q=q, **fields)
 
     @classmethod
-    def get_or_create(cls, safe=False, **fields):
+    def get_or_create(cls, **fields):
         """Gets an existing or creates a new document.
 
         This will find and return a single document matching the
@@ -384,9 +257,11 @@ class Model(object):
         will also be returned to indicate whether or not the document
         was created.
 
-        :param safe: (optional) Whether to perform the create in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :param \*\*fields: Keyword arguments specifying the query.
         :type \*\*fields: \*\*kwargs.
         :returns: tuple -- the :class:`~simon.Model` and whether the
@@ -396,12 +271,18 @@ class Model(object):
         .. versionadded:: 0.1.0
         """
 
+        write_concern = {
+            'safe': fields.pop('safe', None),
+            'w': fields.pop('w', None),
+        }
+
         try:
             return cls.get(**fields), False
         except cls.NoDocumentFound:
-            return cls.create(safe=safe, **fields), True
+            return cls.create(safe=write_concern['safe'], w=write_concern['w'],
+                              **fields), True
 
-    def increment(self, field=None, value=1, safe=False, **fields):
+    def increment(self, field=None, value=1, **fields):
         """Performs an atomic increment.
 
         This can be used to update a single field::
@@ -426,9 +307,11 @@ class Model(object):
         :type field: str.
         :param value: (optional) Value to increment ``field`` by.
         :type value: int.
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :param \*\*fields: Keyword arguments specifying fields and
                            increment values.
         :type \*\*fields: \*\*kwargs.
@@ -437,6 +320,11 @@ class Model(object):
         .. versionadded:: 0.1.0
         """
 
+        write_concern = {
+            'safe': fields.pop('safe', None),
+            'w': fields.pop('w', None),
+        }
+
         # There needs to be something to update.
         if field is None and not fields:
             raise ValueError('No fields have been specified.')
@@ -451,9 +339,9 @@ class Model(object):
         for k, v in fields.items():
             update[k] = v
 
-        self._update({'$inc': update}, safe=safe)
+        self._update({'$inc': update}, **write_concern)
 
-    def pop(self, fields, safe=False):
+    def pop(self, fields, **kwargs):
         """Performs an atomic pop.
 
         Values can be popped from either the end or the beginning of a
@@ -467,9 +355,11 @@ class Model(object):
 
         :param fields: The names of the fields to pop from.
         :type fields: str, list, or tuple.
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :raises: :class:`TypeError`
 
         .. versionadded:: 0.5.0
@@ -478,6 +368,11 @@ class Model(object):
         if not fields:
             raise ValueError
 
+        write_concern = {
+            'safe': kwargs.pop('safe', None),
+            'w': kwargs.pop('w', None),
+        }
+
         if not isinstance(fields, (list, tuple)):
             fields = (fields,)
 
@@ -492,9 +387,9 @@ class Model(object):
 
             update[field] = direction
 
-        self._update({'$pop': update}, safe=safe)
+        self._update({'$pop': update}, **write_concern)
 
-    def pull(self, field=None, value=None, safe=False, **fields):
+    def pull(self, field=None, value=None, **fields):
         """Performs an atomic pull.
 
         With MongoDB there are two types of pull operations: ``$pull``
@@ -525,9 +420,11 @@ class Model(object):
         :type field: str.
         :param value: (optional) Value to pull from ``field``.
         :type value: scalar or list.
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :param \*\*fields: Keyword arguments specifying fields and
                            the values to pull.
         :type \*\*fields: \*\*kwargs.
@@ -536,6 +433,11 @@ class Model(object):
         .. versionadded:: 0.5.0
         """
 
+        write_concern = {
+            'safe': fields.pop('safe', None),
+            'w': fields.pop('w', None),
+        }
+
         # There needs to be something to update.
         if not (field and value) and not fields:
             raise ValueError('No fields have been specified.')
@@ -560,10 +462,9 @@ class Model(object):
             else:
                 update['$pull'][k] = v
 
-        self._update(update, safe=safe)
+        self._update(update, **write_concern)
 
-    def push(self, field=None, value=None, allow_duplicates=True, safe=False,
-             **fields):
+    def push(self, field=None, value=None, allow_duplicates=True, **fields):
         """Performs an atomic push.
 
         With MongoDB there are three types of push operations:
@@ -601,9 +502,11 @@ class Model(object):
         :param allow_duplicates: (optional) Whether to allow duplicate
                                  values to be added to the list
         :type allow_duplicates: bool.
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :param \*\*fields: Keyword arguments specifying fields and
                            the values to push.
         :type \*\*fields: \*\*kwargs.
@@ -612,6 +515,11 @@ class Model(object):
         .. versionadded:: 0.5.0
         """
 
+        write_concern = {
+            'safe': fields.pop('safe', None),
+            'w': fields.pop('w', None),
+        }
+
         # There needs to be something to update.
         if not (field and value) and not fields:
             raise ValueError('No fields have been specified.')
@@ -639,9 +547,9 @@ class Model(object):
             else:
                 update['$push' if allow_duplicates else '$addToSet'][k] = v
 
-        self._update(update, safe=safe)
+        self._update(update, **write_concern)
 
-    def raw_update(self, fields, safe=False):
+    def raw_update(self, fields, **kwargs):
         """Performs an update using a raw document.
 
         This method should be used carefully as it will perform the
@@ -662,17 +570,24 @@ class Model(object):
 
         :param fields: The document to save to the database.
         :type fields: dict.
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :raises: :class:`TypeError`
 
         .. versionadded:: 0.1.0
         """
 
-        self._update(fields, safe=safe)
+        write_concern = {
+            'safe': kwargs.pop('safe', None),
+            'w': kwargs.pop('w', None),
+        }
 
-    def remove_fields(self, fields, safe=False):
+        self._update(fields, **write_concern)
+
+    def remove_fields(self, fields, **kwargs):
         """Removes the specified fields from the document.
 
         The specified fields will be removed from the document in the
@@ -692,14 +607,21 @@ class Model(object):
 
         :param fields: The names of the fields to remove.
         :type fields: str, list, or tuple.
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :raises: :class:`TypeError`
 
         .. versionadded:: 0.1.0
         """
 
+        write_concern = {
+            'safe': kwargs.pop('safe', None),
+            'w': kwargs.pop('w', None),
+        }
+
         # fields can contain a single item as a string. If it's not a
         # list or tuple, make it one. Otherwise the generators below
         # would iterate over each character in the string rather than
@@ -709,9 +631,9 @@ class Model(object):
 
         fields = dict((k, 1) for k in fields)
 
-        self._update({'$unset': fields}, safe=safe)
+        self._update({'$unset': fields}, **write_concern)
 
-    def rename(self, field_from=None, field_to=None, safe=False, **fields):
+    def rename(self, field_from=None, field_to=None, **fields):
         """Performs an atomic rename.
 
         This can be used to update a single field::
@@ -737,9 +659,11 @@ class Model(object):
         :type field_from: str.
         :param field_to: (optional) New name for ``field_from``.
         :type field_to: int.
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :param \*\*fields: Keyword arguments specifying fields and their
                            new names.
         :type \*\*fields: \*\*kwargs.
@@ -748,6 +672,11 @@ class Model(object):
         .. versionadded:: 0.5.0
         """
 
+        write_concern = {
+            'safe': fields.pop('safe', None),
+            'w': fields.pop('w', None),
+        }
+
         # There needs to be something to update.
         if not (field_from and field_to) and not fields:
             raise ValueError('No fields have been specified.')
@@ -762,9 +691,9 @@ class Model(object):
         for k, v in fields.items():
             update[k] = v
 
-        self._update({'$rename': update}, safe=safe)
+        self._update({'$rename': update}, **write_concern)
 
-    def save(self, safe=False):
+    def save(self, **kwargs):
         """Saves the object to the database.
 
         When saving a new document for a model with ``auto_timestamp``
@@ -776,9 +705,11 @@ class Model(object):
         :class:`TypeError` will be raised if any of the fields have not
         been associated with the instance.
 
-        :param safe: (optional) Whether to perform the save in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :raises: :class:`TypeError`
 
         .. versionchanged:: 0.4.0
@@ -788,6 +719,11 @@ class Model(object):
         .. versionadded:: 0.1.0
         """
 
+        write_concern = {
+            'safe': kwargs.pop('safe', None),
+            'w': kwargs.pop('w', None),
+        }
+
         # Use a copy of the internal document so _id can be safely
         # removed.
         fields = self._document.copy()
@@ -806,7 +742,7 @@ class Model(object):
 
         try:
             # Use upsert=True so new documents can be inserted.
-            self._update(fields, upsert=True, safe=safe)
+            self._update(fields, upsert=True, **write_concern)
         except:
             # Raise the exception that was caught
             e = sys.exc_info()
@@ -821,7 +757,7 @@ class Model(object):
                 if 'modified' in fields:
                     self._document['modified'] = fields['modified']
 
-    def save_fields(self, fields, safe=False):
+    def save_fields(self, fields, **kwargs):
         """Saves only the specified fields.
 
         If only a select number of fields need to be updated, an atomic
@@ -844,14 +780,21 @@ class Model(object):
 
         :param fields: The names of the fields to update.
         :type fields: str, list, or tuple.
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :raises: :class:`AttributeError`, :class:`TypeError`
 
         .. versionadded:: 0.1.0
         """
 
+        write_concern = {
+            'safe': kwargs.pop('safe', None),
+            'w': kwargs.pop('w', None),
+        }
+
         # fields can contain a single item as a string. If it's not a
         # list or tuple, make it one. Otherwise the generators below
         # would iterate over each character in the string rather than
@@ -863,9 +806,9 @@ class Model(object):
 
         # Set use_internal so that the real values will come from
         # self._document instead of the document passed to _update().
-        self._update({'$set': update}, use_internal=True, safe=safe)
+        self._update({'$set': update}, use_internal=True, **write_concern)
 
-    def update(self, safe=False, **fields):
+    def update(self, **fields):
         """Performs an atomic update.
 
         If only a select number of fields need to be updated, an atomic
@@ -881,9 +824,11 @@ class Model(object):
         Unlike :meth:`~simon.Model.save`, ``modified`` will not be
         updated.
 
-        :param safe: (optional) Whether to perform the update in safe
-                     mode.
+        :param safe: (optional) **DEPRECATED** Use ``w`` instead.
         :type safe: bool.
+        :param w: (optional) The number of servers that must receive the
+                  update for it to be successful.
+        :type w: int.
         :param \*\*fields: The fields to update.
         :type \*\*fields: \*\*kwargs.
         :raises: :class:`TypeError`
@@ -891,7 +836,12 @@ class Model(object):
         .. versionadded:: 0.1.0
         """
 
-        self._update({'$set': fields}, safe=safe)
+        write_concern = {
+            'safe': fields.pop('safe', None),
+            'w': fields.pop('w', None),
+        }
+
+        self._update({'$set': fields}, **write_concern)
 
     # Database interaction methods
 
@@ -924,6 +874,9 @@ class Model(object):
         :raises: :class:`~simon.Model.MultipleDocumentsFound`,
                  :class:`~simon.Model.NoDocumentFound`
 
+        .. versionchanged:: 0.6.0
+           ``_id`` can be a type other than :class:`~pymongo.ObjectId`
+
         .. versionadded:: 0.3.0
         """
 
@@ -931,10 +884,12 @@ class Model(object):
         if isinstance(q, Q):
             fields.update(q._filter)
 
-        query = map_fields(cls, fields, flatten_keys=True, with_operators=True)
+        query = map_fields(cls._meta.field_map, fields, flatten_keys=True,
+                           with_operators=True)
 
-        # If querying by the _id, make sure it's an Object ID.
-        if '_id' in query:
+        # If querying by the _id, make sure it's an Object ID, but only
+        # if it's typed as one.
+        if '_id' in query and cls._meta.typed_fields['_id'] == ObjectId:
             query['_id'] = guarantee_object_id(query['_id'])
 
         # Find all of the matching documents.
@@ -1002,7 +957,12 @@ class Model(object):
         :type use_internal: bool.
         :param \*\*kwargs: The optional settings.
         :type \*\*kwargs: \*\*kwargs.
-        :raises: :class:`TypeError`
+        :raises: :class:`AttributeError`, :class:`TypeError`
+
+        .. versionchanged:: 0.6.0
+           ``typed_fields`` is enforced
+           ``_id`` can be a type other than :class:`~pymongo.ObjectId`
+           ``safe`` is being deprecated in favor of ``w`` (PyMongo 2.4+)
 
         .. versionadded:: 0.3.0
         """
@@ -1010,11 +970,50 @@ class Model(object):
         # Save characters
         cls = self.__class__
 
+        def check_typed_fields(fields):
+            """Checks that fields are of the correct type.
+
+            This function checks the fields that are being saved and
+            makes sure they are of the correct type. If a field isn't
+            of the right type, :class:`TypeError` will be raised.
+
+            :param fields: The document to check.
+            :type fields: dict.
+            :raises: :class:`TypeError`
+
+            .. versionadded:: 0.6.0
+            """
+
+            for k, v in cls._meta.typed_fields.items():
+                if v is None:
+                    # None means the field can be any type. This will
+                    # probably be used most often with _id.
+                    continue
+
+                try:
+                    value = get_nested_key(fields, k)
+                except KeyError:
+                    # Nothing to see here
+                    continue
+
+                if isinstance(v, list):
+                    if all(isinstance(x, v[0]) for x in value):
+                        # All values in the list are of the right type.
+                        continue
+                elif isinstance(value, v):
+                    # The value is of the right type.
+                    continue
+
+                message = ("The '{0}' object cannot be updated because "
+                           "its '{1}' field must be {2}.")
+                message = message.format(cls.__name__, k, v)
+                raise TypeError(message)
+
         def map_field_names_and_values(fields):
             """Maps field names and values.
 
-            This method will take care of mapping the keys of ``fields``
-            to the correct syntax for the query.
+            This function will take care of mapping the keys of
+            ``fields`` to the correct syntax for the query.
 
             When the  ``use_internal`` argument of
             :meth:`~simon.Model._update()` is set to ``True``, it will
@@ -1029,11 +1028,11 @@ class Model(object):
             .. versionadded:: 0.3.0
             """
 
-            fields = map_fields(cls, fields, flatten_keys=True)
+            fields = map_fields(cls._meta.field_map, fields, flatten_keys=True)
             if use_internal:
                 try:
                     fields = dict((k, get_nested_key(self._document, k))
-                                  for k in fields.keys())
+                                  for k in fields.iterkeys())
                 except KeyError:
                     # KeyError will be raised by get_nested_key() when a
                     # field isn't part of the internal document.
@@ -1049,10 +1048,13 @@ class Model(object):
                        "attribute has not been set.".format(cls.__name__))
             raise TypeError(message)
 
-        # Capture the _id and make sure it's an Object ID
+        # Capture the _id.
         id = self._document.get('_id')
         if id:
-            id = guarantee_object_id(id)
+            # Make sure it's an Object ID, but only if it's typed as
+            # one.
+            if cls._meta.typed_fields['_id'] == ObjectId:
+                id = guarantee_object_id(id)
             # Updates (as opposed to inserts) need spec to match against
             kwargs['spec'] = {'_id': id}
 
@@ -1062,7 +1064,7 @@ class Model(object):
                 fields[k] = map_field_names_and_values(v)
                 if k == '$rename':
                     for field_from, field_to in fields[k].items():
-                        mapped = map_fields(cls, {field_to: 1},
+                        mapped = map_fields(cls._meta.field_map, {field_to: 1},
                                             flatten_keys=True)
                         fields[k][field_from] = mapped.keys()[0]
         else:
@@ -1100,18 +1102,47 @@ class Model(object):
                                          ', '.join(self._meta.required_fields))
                 raise TypeError(message)
 
+        # Enforce the typed fields
+        if cls._meta.typed_fields:
+            if is_atomic(fields):
+                for k, v in fields.items():
+                    if k == '$rename':
+                        # $rename doesn't alter the value stored in the
+                        # database. Therefore the value associated with
+                        # the instance is not a reliable way to tell if
+                        # a renamed field will be of the wrong type. The
+                        # only way to really know would be to load the
+                        # value from the database. Rather than incurring
+                        # this blocking operation, renaming to a typed
+                        # field will trigger a UserWarning instead.
+                        if any(v in cls._meta.typed_fields
+                               for v in v.itervalues()):
+                            message = ('You are renaming a typed field. Its '
+                                       'value may not be of the correct type.')
+                            warnings.warn(message, UserWarning)
+                    elif k != '$unset':
+                        # Fields don't need to be checked if they're
+                        # being removed.
+                        check_typed_fields(v)
+            else:
+                check_typed_fields(fields)
+
         # Handling the write concern argument has been pushed off to
         # another method that is aware of what PyMongo supports.
-        _set_write_concern(kwargs, self._meta.safe)
+        set_write_concern(kwargs, self._meta.write_concern)
 
         # Which function are we calling?
         if not id:
             # PyMongo's insert() method calls the argument doc_or_docs
             # instead of document.
             kwargs['doc_or_docs'] = kwargs.pop('document')
-            f = self._meta.db.insert
+            f = cls._meta.db.insert
         else:
-            f = self._meta.db.update
+            if cls._meta.typed_fields['_id'] != ObjectId:
+                # In order to handle inserting documents with custom
+                # values for _id an upsert is needed.
+                kwargs['upsert'] = True
+            f = cls._meta.db.update
 
         result = f(**kwargs)
 
@@ -1129,7 +1160,7 @@ class Model(object):
             # that have been updated. Build a dictionary containing the
             # keys of the fields that need to be reloaded and retrieve
             # them from the database. Then apply the new values to the
-            # instance's interal document.
+            # instance's internal document.
 
             unset = fields.pop('$unset', None)
             if unset:
@@ -1148,18 +1179,19 @@ class Model(object):
             if rename:
                 # The old fields need to be removed. The new fields will
                 # be added through the find_one() below.
-                for k in rename.keys():
+                for k in rename.iterkeys():
                     if '.' in k:
                         self._document = remove_nested_key(self._document, k)
                     else:
                         self._document.pop(k, None)
-                fields['$rename'] = dict((v, 1) for v in rename.values())
+                fields['$rename'] = dict((v, 1) for v in rename.itervalues())
 
             # If the only operation was an $unset, we're done.
             if not fields:
                 return
 
-            fields = dict((k, 1) for v in fields.values() for k in v.keys())
+            fields = dict((k, 1) for v in fields.itervalues()
+                          for k in v.iterkeys())
             doc = self._meta.db.find_one({'_id': id}, fields)
 
             # There's no need to update the _id
@@ -1212,8 +1244,8 @@ class Model(object):
                 # map_fields() requires a dict, so make a simple one and
                 # then capture the first (only) key in the resulting
                 # dict.
-                mapped_name = map_fields(self.__class__, {name: 1},
-                                         flatten_keys=True)
+                mapped_name = map_fields(self.__class__._meta.field_map,
+                                         {name: 1}, flatten_keys=True)
                 mapped_name = mapped_name.keys()[0]
             else:
                 mapped_name = name
@@ -1248,7 +1280,7 @@ class Model(object):
             # map_fields() requires a dict, so make a simple one and
             # then capture the first (only) key in the resulting
             # dict.
-            mapped_name = map_fields(self.__class__, {name: 1},
+            mapped_name = map_fields(self.__class__._meta.field_map, {name: 1},
                                      flatten_keys=True)
             mapped_name = mapped_name.keys()[0]
 
