@@ -9,12 +9,15 @@ compatability of its API.
 
 import collections
 from datetime import datetime
+import warnings
 
 from bson import ObjectId
 
+from .connection import pymongo_supports_mongoclient
+
 __all__ = ('current_datetime', 'get_nested_key', 'guarantee_object_id',
            'is_atomic', 'map_fields', 'parse_kwargs', 'remove_nested_key',
-           'update_nested_keys')
+           'set_write_concern', 'update_nested_keys')
 
 
 # The logical operators are needed when mapping fields. The values
@@ -147,14 +150,14 @@ def is_atomic(document):
     .. versionadded:: 0.3.0
     """
 
-    return any(k[0] == '$' for k in document.keys())
+    return any(k[0] == '$' for k in document.iterkeys())
 
 
-def map_fields(cls, fields, with_operators=False, flatten_keys=False):
+def map_fields(field_map, fields, with_operators=False, flatten_keys=False):
     """Maps attribute names to document keys.
 
-    Attribute names will be mapped to document keys using
-    ``cls._meta.field_map``. If any of the attribute names contain
+    Attribute names will be mapped to document keys using the mapping
+    specified in ``field_map``. If any of the attribute names contain
     ``__``, :meth:`parse_kwargs` will be called and a second pass
     through ``cls._meta.field_map`` will be performed.
 
@@ -191,7 +194,7 @@ def map_fields(cls, fields, with_operators=False, flatten_keys=False):
     operator sans the ``$`` (e.g., ``__gt``, ``__lt``) to the name of
     the key::
 
-        map_fields(ModelClass, {'a__gt': 1, 'b__lt': 2},
+        map_fields(mapping, {'a__gt': 1, 'b__lt': 2},
                    with_operators=True)
 
     This will check for a greater than 1 and b less than 2 as::
@@ -201,7 +204,7 @@ def map_fields(cls, fields, with_operators=False, flatten_keys=False):
     The ``$not`` operator can be used in conjunction with any of the
     above operators::
 
-        map_fields(ModelClass, {'a__gt': 1, 'b__not__lt': 2},
+        map_fields(mapping, {'a__gt': 1, 'b__not__lt': 2},
                    with_operators=True)
 
     This will check for a greater than 1 and b not less than 2 as::
@@ -212,8 +215,8 @@ def map_fields(cls, fields, with_operators=False, flatten_keys=False):
     of the result dictionary, using a ``.`` to separate each part of a
     key. When this happens, the second pass will be omitted.
 
-    :param cls: A subclass of :class:`~simon.Model`.
-    :type cls: type.
+    :param field_map: Key/value pairs defining the field map.
+    :type field_map: dict.
     :param fields: Key/value pairs to be used for queries.
     :type fields: dict.
     :param with_operators: (optional) Whether or not to process
@@ -269,7 +272,7 @@ def map_fields(cls, fields, with_operators=False, flatten_keys=False):
             # back into map_fields() and a new list should be build
             # from the mapped dictionaries.
             if isinstance(v, list):
-                v = [map_fields(cls=cls, fields=x,
+                v = [map_fields(field_map=field_map, fields=x,
                                 with_operators=with_operators,
                                 flatten_keys=flatten_keys) for x in v]
         else:
@@ -279,7 +282,7 @@ def map_fields(cls, fields, with_operators=False, flatten_keys=False):
             # the syntax for mapping embedded keys. If a . exists in the
             # new key, second_pass should be set to True because
             # parse_kwargs() should be run
-            k = cls._meta.field_map.get(k.replace('__', '.'), k)
+            k = field_map.get(k.replace('__', '.'), k)
             if '.' in k:
                 second_pass = True
 
@@ -311,7 +314,7 @@ def map_fields(cls, fields, with_operators=False, flatten_keys=False):
 
         mapped_fields = {}
         for k, v in fields.items():
-            k = cls._meta.field_map.get(k, k)
+            k = field_map.get(k, k)
 
             mapped_fields[k] = v
 
@@ -399,6 +402,71 @@ def remove_nested_key(original, key):
     return original
 
 
+def set_write_concern_as_safe(options, force_write_concern):
+    """Sets the safe parameter for write concern.
+
+    If ``force_write_concern`` is ``True``, write concern will be used.
+
+    :param options: The potential write concern settings.
+    :type options: dict.
+    :param force_write_concern: A value to override ``options`` with.
+    :type force_write_concern: bool.
+
+    .. versionadded:: 0.6.0
+    """
+
+    safe = options.pop('safe', None)
+    w = options.pop('w', None)
+
+    if force_write_concern:
+        safe = True
+    else:
+        if safe is None and w:
+            safe = True
+        else:
+            safe = safe or False
+    options['safe'] = safe
+
+
+def set_write_concern_as_w(options, force_write_concern):
+    """Sets the w parameter for write concern.
+
+    If ``force_write_concern`` is a value greater than zero, write
+    concern will be used.
+
+    :param options: The potential write concern settings.
+    :type options: dict.
+    :param force_write_concern: A value to override ``options`` with.
+    :type force_write_concern: int.
+
+    .. versionadded:: 0.6.0
+    """
+
+    if 'safe' in options and options['safe'] is not None:
+        message = 'safe has been deprecated. Please use w instead.'
+        warnings.warn(message, DeprecationWarning)
+
+    safe = options.pop('safe', None)
+    w = options.pop('w', None)
+
+    if force_write_concern:
+        # use int() in case force_write_concern is True
+        w = int(force_write_concern)
+    else:
+        if w is None and safe:
+            w = 1
+        else:
+            w = w or 0
+    options['w'] = w
+
+
+# Export the relevant function as of set_write_concern()
+if pymongo_supports_mongoclient:
+    set_write_concern = set_write_concern_as_w
+else:
+    set_write_concern = set_write_concern_as_safe
+
+
 def update_nested_keys(original, updates):
     """Updates keys within nested dictionaries.
 
