@@ -27,6 +27,13 @@ class TestModel(Model):
         database = 'simon-integration'
 
 
+class TestUntypedModel(Model):
+    class Meta:
+        collection = 'simon-integration'
+        database = 'simon-integration'
+        typed_fields = {'_id': None}
+
+
 class TestConnectionIntegrations(unittest.TestCase):
     """Connection integration tests"""
 
@@ -327,6 +334,18 @@ class TestDatabaseIntegrations(unittest.TestCase):
 
         self.assertEqual(doc['b'], 2)
 
+        # Update with non-Object Id
+
+        self.collection.insert({'_id': 1})
+
+        m = TestUntypedModel(_id=1)
+        m.a = 1
+        m.save()
+
+        doc = self.collection.find_one({'_id': 1})
+
+        self.assertEqual(doc['a'], 1)
+
     def test_save_fields(self):
         """Test the `save_fields()` method."""
 
@@ -340,6 +359,23 @@ class TestDatabaseIntegrations(unittest.TestCase):
 
         self.assertEqual(doc['b'], 2)
 
+    def test_sort(self):
+        """Test the `sort()` method."""
+
+        self.collection.insert([{'a': 2}, {'a': 1}, {'a': 3}])
+
+        qs = TestModel.all().sort('a')
+
+        self.assertEqual(qs[0]._document['a'], 1)
+        self.assertEqual(qs[1]._document['a'], 2)
+        self.assertEqual(qs[2]._document['a'], 3)
+
+        qs = TestModel.all().sort('-a')
+
+        self.assertEqual(qs[0]._document['a'], 3)
+        self.assertEqual(qs[1]._document['a'], 2)
+        self.assertEqual(qs[2]._document['a'], 1)
+
     def test_update(self):
         """Test the `update()` method."""
 
