@@ -17,13 +17,31 @@ from simon.query import Q
 from .utils import AN_OBJECT_ID, ModelFactory
 
 DefaultModel = ModelFactory('DefaultModel')
-MappedModel = ModelFactory('ModelFactory', field_map={'fake': 'real'})
+MappedModel = ModelFactory('MappedModel', field_map={'fake': 'real'})
 StringModel = ModelFactory('StringModel',
                            __str__=lambda x: 'this is the __str__',
                            __unicode__=lambda x: 'this is the __unicode__')
 
 
-class TestBase(unittest.TestCase):
+class BaseModel(Model):
+    """Use for testing subclassing."""
+
+    class Meta:
+        auto_timestamp = False
+        collection = 'collection'
+        database = 'database'
+        field_map = {'fake': 'real'}
+        map_id = False
+        required_fields = 'a'
+        safe = False
+        sort = 'a'
+
+
+class SubclassModel(BaseModel):
+    """Use for testing subclassing."""
+
+
+class TestModel(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         with mock.patch('simon.connection.MongoClient'):
@@ -73,7 +91,8 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(TestModel, '_update') as _update:
             TestModel.create(a=1)
 
-            _update.assert_called_with({'a': 1}, safe=False, upsert=True)
+            _update.assert_called_with({'a': 1}, safe=None, w=None,
+                                       upsert=True)
 
     def test_db(self):
         ("Test that the `db` attribute is associated with classes and "
@@ -225,7 +244,7 @@ class TestBase(unittest.TestCase):
             # Because get_or_create() is being called without explicity
             # setting a value for safe, safe's default value will be
             # passed along to creates()
-            create.assert_called_with(_id=AN_OBJECT_ID, safe=False)
+            create.assert_called_with(_id=AN_OBJECT_ID, safe=None, w=None)
 
             self.assertTrue(created)
 
@@ -282,11 +301,11 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.increment('a')
 
-            _update.assert_called_with({'$inc': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$inc': {'a': 1}}, safe=None, w=None)
 
             m.increment('a', 2)
 
-            _update.assert_called_with({'$inc': {'a': 2}}, safe=False)
+            _update.assert_called_with({'$inc': {'a': 2}}, safe=None, w=None)
 
     def test_increment_multiple(self):
         """Test the `increment()` method with multiple fields."""
@@ -296,7 +315,8 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.increment(a=1, b=2)
 
-            _update.assert_called_with({'$inc': {'a': 1, 'b': 2}}, safe=False)
+            _update.assert_called_with({'$inc': {'a': 1, 'b': 2}}, safe=None,
+                                       w=None)
 
     def test_increment_valueerror(self):
         """Test that `increment()` raises `ValueError`."""
@@ -349,15 +369,15 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.pop('a')
 
-            _update.assert_called_with({'$pop': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$pop': {'a': 1}}, safe=None, w=None)
 
             m.pop(['a'])
 
-            _update.assert_called_with({'$pop': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$pop': {'a': 1}}, safe=None, w=None)
 
             m.pop('-a')
 
-            _update.assert_called_with({'$pop': {'a': -1}}, safe=False)
+            _update.assert_called_with({'$pop': {'a': -1}}, safe=None, w=None)
 
     def test_pop_multiple(self):
         """Test the `pop()` method with multiple fields."""
@@ -367,16 +387,18 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.pop(('a', 'b'))
 
-            _update.assert_called_with({'$pop': {'a': 1, 'b': 1}}, safe=False)
+            _update.assert_called_with({'$pop': {'a': 1, 'b': 1}}, safe=None,
+                                       w=None)
 
             m.pop(('a', '-b'))
 
-            _update.assert_called_with({'$pop': {'a': 1, 'b': -1}}, safe=False)
+            _update.assert_called_with({'$pop': {'a': 1, 'b': -1}}, safe=None,
+                                       w=None)
 
             m.pop(('-a', '-b'))
 
             _update.assert_called_with({'$pop': {'a': -1, 'b': -1}},
-                                       safe=False)
+                                       safe=None, w=None)
 
     def test_pull(self):
         """Test the `pull()` method."""
@@ -386,19 +408,21 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.pull('a', 1)
 
-            _update.assert_called_with({'$pull': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$pull': {'a': 1}}, safe=None, w=None)
 
             m.pull('a', [1, 2])
 
-            _update.assert_called_with({'$pullAll': {'a': [1, 2]}}, safe=False)
+            _update.assert_called_with({'$pullAll': {'a': [1, 2]}}, safe=None,
+                                       w=None)
 
             m.pull(a=2)
 
-            _update.assert_called_with({'$pull': {'a': 2}}, safe=False)
+            _update.assert_called_with({'$pull': {'a': 2}}, safe=None, w=None)
 
             m.pull(a=[2, 3])
 
-            _update.assert_called_with({'$pullAll': {'a': [2, 3]}}, safe=False)
+            _update.assert_called_with({'$pullAll': {'a': [2, 3]}}, safe=None,
+                                       w=None)
 
     def test_pull_multiple(self):
         """Test the `pull()` method with multiple fields."""
@@ -408,19 +432,20 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.pull(a=1, b=2)
 
-            _update.assert_called_with({'$pull': {'a': 1, 'b': 2}}, safe=False)
+            _update.assert_called_with({'$pull': {'a': 1, 'b': 2}}, safe=None,
+                                       w=None)
 
             m.pull(a=1, b=[2, 3])
 
             _update.assert_called_with({'$pull': {'a': 1},
                                         '$pullAll': {'b': [2, 3]}},
-                                       safe=False)
+                                       safe=None, w=None)
 
             m.pull(a=[1, 2], b=[3, 4])
 
             _update.assert_called_with({'$pullAll': {'a': [1, 2],
                                                      'b': [3, 4]}},
-                                       safe=False)
+                                       safe=None, w=None)
 
     def test_pull_valueerror(self):
         """Test that `pull()` raises `ValueError`."""
@@ -441,19 +466,21 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.push('a', 1)
 
-            _update.assert_called_with({'$push': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$push': {'a': 1}}, safe=None, w=None)
 
             m.push('a', [1, 2])
 
-            _update.assert_called_with({'$pushAll': {'a': [1, 2]}}, safe=False)
+            _update.assert_called_with({'$pushAll': {'a': [1, 2]}}, safe=None,
+                                       w=None)
 
             m.push(a=2)
 
-            _update.assert_called_with({'$push': {'a': 2}}, safe=False)
+            _update.assert_called_with({'$push': {'a': 2}}, safe=None, w=None)
 
             m.push(a=[2, 3])
 
-            _update.assert_called_with({'$pushAll': {'a': [2, 3]}}, safe=False)
+            _update.assert_called_with({'$pushAll': {'a': [2, 3]}}, safe=None,
+                                       w=None)
 
     def tet_push_addtoset(self):
         """Test the `push()` method with `$addToSet`."""
@@ -463,7 +490,8 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.push('a', 1, allow_duplicates=False)
 
-            _update.assert_called_with({'$addToSet': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$addToSet': {'a': 1}}, safe=None,
+                                       w=None)
 
             m.push('a', [1, 2], allow_duplicates=False)
 
@@ -472,12 +500,13 @@ class TestBase(unittest.TestCase):
 
             m.push(a=2, allow_duplicates=False)
 
-            _update.assert_called_with({'$addToSet': {'a': 2}}, safe=False)
+            _update.assert_called_with({'$addToSet': {'a': 2}}, safe=None,
+                                       w=None)
 
             m.push(a=[2, 3], allow_duplicates=False)
 
             _update.assert_called_with({'$addToSet': {'a': {'$each': [2, 3]}}},
-                                       safe=False)
+                                       safe=None, w=None)
 
     def test_push_multiple(self):
         """Test the `push()` method with multiple fields."""
@@ -487,19 +516,20 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.push(a=1, b=2)
 
-            _update.assert_called_with({'$push': {'a': 1, 'b': 2}}, safe=False)
+            _update.assert_called_with({'$push': {'a': 1, 'b': 2}}, safe=None,
+                                       w=None)
 
             m.push(a=1, b=[2, 3])
 
             _update.assert_called_with({'$push': {'a': 1},
                                         '$pushAll': {'b': [2, 3]}},
-                                       safe=False)
+                                       safe=None, w=None)
 
             m.push(a=[1, 2], b=[3, 4])
 
             _update.assert_called_with({'$pushAll': {'a': [1, 2],
                                                      'b': [3, 4]}},
-                                       safe=False)
+                                       safe=None, w=None)
 
     def test_push_multiple_addto_set(self):
         ("Test the `push()` method with multiple fields with "
@@ -511,19 +541,19 @@ class TestBase(unittest.TestCase):
             m.push(a=1, b=2, allow_duplicates=False)
 
             _update.assert_called_with({'$addToSet': {'a': 1, 'b': 2}},
-                                       safe=False)
+                                       safe=None, w=None)
 
             m.push(a=1, b=[2, 3], allow_duplicates=False)
 
             _update.assert_called_with({'$addToSet': {'a': 1,
                                                       'b': {'$each': [2, 3]}}},
-                                       safe=False)
+                                       safe=None, w=None)
 
             m.push(a=[1, 2], b=[3, 4], allow_duplicates=False)
 
             _update.assert_called_with({'$addToSet': {'a': {'$each': [1, 2]},
                                                       'b': {'$each': [3, 4]}}},
-                                       safe=False)
+                                       safe=None, w=None)
 
     def test_push_valueerror(self):
         """Test that `push()` raises `ValueError`."""
@@ -544,7 +574,7 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.raw_update({'$set': {'a': 1}})
 
-            _update.assert_called_with({'$set': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$set': {'a': 1}}, safe=None, w=None)
 
     def test_remove_fields(self):
         """Test the `remove_fields()` method."""
@@ -554,7 +584,7 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.remove_fields('a')
 
-            _update.assert_called_with({'$unset': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$unset': {'a': 1}}, safe=None, w=None)
 
     def test_remove_fields_multiple(self):
         """Test the `remove_fields()` method with multiple fields."""
@@ -565,7 +595,7 @@ class TestBase(unittest.TestCase):
             m.remove_fields(('a', 'b'))
 
             _update.assert_called_with({'$unset': {'a': 1, 'b': 1}},
-                                       safe=False)
+                                       safe=None, w=None)
 
     def test_rename(self):
         """Test the `rename()` method."""
@@ -575,7 +605,8 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.rename('a', 'b')
 
-            _update.assert_called_with({'$rename': {'a': 'b'}}, safe=False)
+            _update.assert_called_with({'$rename': {'a': 'b'}}, safe=None,
+                                       w=None)
 
     def test_rename_multiple(self):
         """Test the `rename()` method with multiple fields."""
@@ -586,7 +617,7 @@ class TestBase(unittest.TestCase):
             m.rename(a='b', c='d')
 
             _update.assert_called_with({'$rename': {'a': 'b', 'c': 'd'}},
-                                       safe=False)
+                                       safe=None, w=None)
 
     def test_rename_valueerror(self):
         """Test that `rename()` raises `ValueError`."""
@@ -627,7 +658,8 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(TestModel, '_update') as _update:
             m.save()
 
-            _update.assert_called_with({'a': 1}, safe=False, upsert=True)
+            _update.assert_called_with({'a': 1}, safe=None, w=None,
+                                       upsert=True)
 
     def test_save_exception(self):
         """Test that `save()` raises the exception it catches."""
@@ -654,7 +686,7 @@ class TestBase(unittest.TestCase):
             m.save_fields('a')
 
             _update.assert_called_with({'$set': {'a': 1}}, use_internal=True,
-                                       safe=False)
+                                       safe=None, w=None)
 
     def test_save_fields_attributeerror(self):
         """Test that `save_fields()` raises `AttributeError`."""
@@ -678,7 +710,7 @@ class TestBase(unittest.TestCase):
             m.save_fields(('a', 'b'))
 
             _update.assert_called_with({'$set': {'a': 1, 'b': 1}},
-                                       use_internal=True, safe=False)
+                                       use_internal=True, safe=None, w=None)
 
     def test_save_timestamps(self):
         """Test that `save()` properly handles adding timestamps."""
@@ -806,4 +838,84 @@ class TestBase(unittest.TestCase):
         with mock.patch.object(DefaultModel, '_update') as _update:
             m.update(a=1)
 
-            _update.assert_called_with({'$set': {'a': 1}}, safe=False)
+            _update.assert_called_with({'$set': {'a': 1}}, safe=None, w=None)
+
+
+class TestModelMetaClass(unittest.TestCase):
+    def test_core_attributes(self):
+        """Test the `_meta.core_attributes` attribute."""
+
+        core_attributes = ('_document', '_meta', 'MultipleDocumentsFound',
+                           'NoDocumentFound')
+
+        self.assertTrue(all(k in DefaultModel._meta.core_attributes
+                            for k in core_attributes))
+        self.assertNotIn('Meta', DefaultModel._meta.core_attributes)
+
+        class AttributeModel(Model):
+            an_attribute = 1
+
+        self.assertIn('an_attribute', AttributeModel._meta.core_attributes)
+
+    def test_db(self):
+        """Test the `_meta.db` attribute."""
+
+        with mock.patch('simon.meta.get_database') as get_database:
+            # Make a new class here to ensure that the database hasn't
+            # yet been set.
+            class MockModel(Model):
+                pass
+
+            # At this point the internal _db should be None.
+            self.assertIsNone(MockModel._meta._db)
+
+            # Once the mock is called, _db should always be set to this
+            # value.
+            get_database.return_value = {MockModel._meta.collection: 1}
+
+            # Accessing the property triggers the call.
+            MockModel._meta.db
+
+            self.assertEqual(MockModel._meta._db, 1)
+            get_database.assert_called_with(MockModel._meta.database)
+
+            # As per above, _db should not get set to this value.
+            get_database.return_value = {MockModel._meta.collection: 2}
+
+            MockModel._meta.db
+
+            self.assertEqual(MockModel._meta._db, 1)
+
+    def test_exceptions(self):
+        """Test that the exceptions are added."""
+
+        self.assertTrue(hasattr(DefaultModel, 'MultipleDocumentsFound'))
+        self.assertTrue(hasattr(DefaultModel, 'NoDocumentFound'))
+
+    def test_subclassed(self):
+        """Test a subclassed model."""
+
+        BaseModel = ModelFactory('BaseModel', auto_timestamp=False,
+                                 collection='collection', database='database',
+                                 field_map={'fake': 'real'}, map_id=False,
+                                 required_fields='a', safe=False, sort='a')
+
+        SubclassModel = ModelFactory('SubclassModel', spec=BaseModel)
+
+        base = BaseModel._meta
+        subclass = SubclassModel._meta
+
+        # All of _meta's attributes should have the same value.
+        self.assertEqual(base.auto_timestamp, subclass.auto_timestamp)
+        self.assertEqual(base.collection, subclass.collection)
+        self.assertEqual(base.database, subclass.database)
+        self.assertEqual(base.field_map, subclass.field_map)
+        self.assertEqual(base.map_id, subclass.map_id)
+        self.assertEqual(base.required_fields, subclass.required_fields)
+        self.assertEqual(base.write_concern, subclass.write_concern)
+        self.assertEqual(base.sort, subclass.sort)
+
+        # Each model should have the same list of core attributes
+        # except that the subclassed one shouldn't have Meta.
+        self.assertEqual(sorted(base.core_attributes),
+                         sorted(subclass.core_attributes))
