@@ -6,7 +6,10 @@ except ImportError:
     import unittest
 
 from contextlib import nested
+import datetime
+import warnings
 
+from bson import ObjectId
 import mock
 import pymongo
 
@@ -25,6 +28,8 @@ else:
 DefaultModel = ModelFactory('DefaultModel')
 MappedModel = ModelFactory('MappedModel', field_map={'fake': 'real'})
 RequiredModel = ModelFactory('RequiredModel', required_fields=('a', 'b'))
+TypedModel = ModelFactory('TypedModel', typed_fields={'a': int})
+TypedListModel = ModelFactory('TypedListModel', typed_fields={'a': [int]})
 
 
 class TestDatabase(unittest.TestCase):
@@ -87,11 +92,11 @@ class TestDatabase(unittest.TestCase):
             # m needs to be given its _id again because delete() strips
             # it to prevent attempts to resave a deleted document
             m = UnsafeModel(_id=AN_OBJECT_ID)
-            m.delete(safe=False)
+            m.delete(w=0)
             remove.assert_called_with({'_id': AN_OBJECT_ID}, **wc_off)
 
             m = UnsafeModel(_id=AN_OBJECT_ID)
-            m.delete(safe=True)
+            m.delete(w=1)
             remove.assert_called_with({'_id': AN_OBJECT_ID}, **wc_on)
 
     def test__find(self):
@@ -180,7 +185,7 @@ class TestDatabase(unittest.TestCase):
             self.assertEqual(actual, expected)
 
     def test__find_nested_field(self):
-        """The the `_find()` method with an embedded document."""
+        """Test the `_find()` method with an embedded document."""
 
         with nested(mock.patch.object(DefaultModel._meta.db, 'find'),
                     mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
@@ -190,6 +195,23 @@ class TestDatabase(unittest.TestCase):
 
             find.assert_called_with({'a.b': 1})
 
+    def test__find_objectid_none(self):
+        """Test the `_find()` method with an untyped Object Id."""
+
+        UntypedModel = ModelFactory('UntypedModel', typed_fields={'_id': None})
+
+        with nested(mock.patch.object(UntypedModel._meta.db, 'find'),
+                    mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
+            find.return_value = QuerySet
+
+            UntypedModel._find(_id='a')
+
+            find.assert_called_with({'_id': 'a'})
+
+            UntypedModel._find(_id=1)
+
+            find.assert_called_with({'_id': 1})
+
     def test__find_objectid_string(self):
         """Test the `_find()` method with a string `_id`."""
 
@@ -201,6 +223,19 @@ class TestDatabase(unittest.TestCase):
 
             find.assert_called_with({'_id': AN_OBJECT_ID})
 
+    def test__find_objectid_typed(self):
+        """Test the `_find()` method with a typed Object Id."""
+
+        IntModel = ModelFactory('IntModel', typed_fields={'_id': int})
+
+        with nested(mock.patch.object(IntModel._meta.db, 'find'),
+                    mock.patch('simon.base.QuerySet'),) as (find, QuerySet):
+            find.return_value = QuerySet
+
+            IntModel._find(_id=1)
+
+            find.assert_called_with({'_id': 1})
+
     def test__find_q(self):
         """Test the `_find()` method with a `Q` object."""
 
@@ -381,6 +416,39 @@ class TestDatabase(unittest.TestCase):
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
                                       document={'a.b': 1}, **wc_on)
 
+    def test__update_objectid_none(self):
+        """Test the `_update()` method with an untyped Object Id."""
+
+        UntypedModel = ModelFactory('UntypedModel', typed_fields={'_id': None})
+
+        with mock.patch.object(UntypedModel._meta.db, 'update') as update:
+            m1 = UntypedModel(_id='a')
+
+            m1._update({'b': 1})
+
+            update.assert_called_with(spec={'_id': 'a'}, document={'b': 1},
+                                      upsert=True, **wc_on)
+
+            m2 = UntypedModel(_id=1)
+
+            m2._update({'b': 2})
+
+            update.assert_called_with(spec={'_id': 1}, document={'b': 2},
+                                      upsert=True, **wc_on)
+
+    def test__update_objectid_typed(self):
+        """Test the `_update()` method with a typed Object Id."""
+
+        IntModel = ModelFactory('IntModel', typed_fields={'_id': int})
+
+        m = IntModel(_id=1)
+
+        with mock.patch.object(IntModel._meta.db, 'update') as update:
+            m._update({'b': 2})
+
+            update.assert_called_with(spec={'_id': 1}, document={'b': 2},
+                                      upsert=True, **wc_on)
+
     def test__update_pop(self):
         """Test the `_update()` method with a pop."""
 
@@ -575,7 +643,7 @@ class TestDatabase(unittest.TestCase):
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
                                       document={'a.b': 1}, **wc_on)
 
-    def test__update_required_rename(self):
+    def test__update_required_field_rename(self):
         ("Test the `_update()` method with a required field with a "
          "rename.")
 
@@ -604,6 +672,161 @@ class TestDatabase(unittest.TestCase):
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
                                       document={'$unset': {'c': 1}}, **wc_on)
 
+    def test__update_typed_field(self):
+        """Test the `_update()` method with a typed field."""
+
+        m = TypedModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(TypedModel._meta.db, 'update') as update:
+            m._update({'a': 1})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'a': 1}, **wc_on)
+
+    def test__update_typed_field_atomic(self):
+        ("Test the `_update()` method with a typed field with an atomic"
+         " update.")
+
+        m = TypedModel(_id=AN_OBJECT_ID)
+
+        with nested(mock.patch.object(TypedModel._meta.db, 'update'),
+                    mock.patch.object(TypedModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': 1}
+
+            m._update({'$set': {'a': 1}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$set': {'a': 1}}, **wc_on)
+
+    def test__update_type_field_list(self):
+        ("Test the `_update()` method with a typed field that is a "
+         "typed list.")
+
+        m = TypedListModel(_id=AN_OBJECT_ID)
+
+        with nested(mock.patch.object(TypedListModel._meta.db, 'update'),
+                    mock.patch.object(TypedListModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': [1]}
+
+            m._update({'a': [1]})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'a': [1]}, **wc_on)
+
+    def test__update_typed_field_nested(self):
+        ("Test the `_update()` method with a typed field with an "
+         "embedded document.")
+
+        TypedEmbeddedModel = ModelFactory('TypedEmbeddedModel',
+                                          typed_fields={'a.b': int})
+
+        m = TypedEmbeddedModel(_id=AN_OBJECT_ID)
+
+        with nested(mock.patch.object(TypedEmbeddedModel._meta.db, 'update'),
+                    mock.patch.object(TypedEmbeddedModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': {'b': 1}}
+
+            m._update({'a.b': 1})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'a.b': 1}, **wc_on)
+
+    def test__update_type_field_none(self):
+        ("Test the `_update()` method with a typed field set to None "
+         "instead of a type.")
+
+        TypedNoneModel = ModelFactory('TypedNoneModel',
+                                      typed_fields={'a': None})
+
+        m = TypedNoneModel(_id=AN_OBJECT_ID)
+
+        with nested(mock.patch.object(TypedNoneModel._meta.db, 'update'),
+                    mock.patch.object(TypedNoneModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': 1}
+
+            m._update({'a': 1})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'a': 1}, **wc_on)
+
+    def test__update_typed_field_rename(self):
+        ("Test the `_update()` method with a typed field with a "
+         "rename.")
+
+        m = TypedModel(_id=AN_OBJECT_ID, b=1)
+
+        # This with needs to go before catch_warnings() or it will a
+        # DeprecationWarning will be included in w for Python 2.7+.
+        with nested(mock.patch.object(TypedModel._meta.db, 'update'),
+                    mock.patch.object(TypedModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': 1}
+
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+
+                m._update({'$rename': {'b': 'a'}})
+
+                update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                          document={'$rename': {'b': 'a'}},
+                                          **wc_on)
+
+            self.assertEqual(len(w), 1)
+            self.assertTrue(issubclass(w[-1].category, UserWarning))
+
+            expected = ('You are renaming a typed field. Its value may not be '
+                        'of the correct type.')
+            actual = str(w[-1].message)
+            self.assertEqual(actual, expected)
+
+    def test__update_typed_field_types(self):
+        ("Test the `_update()` method with different types of typed "
+         "fields.")
+
+        MultipleTypedModel = ModelFactory('MultipleTypedModel',
+                                          typed_fields={
+                                              'int': int,
+                                              'string': basestring,
+                                              'datetime': datetime.datetime,
+                                              'list': list,
+                                              'objectid': ObjectId,
+                                          })
+
+        m = MultipleTypedModel(_id=AN_OBJECT_ID)
+
+        # Test a successful update.
+        with nested(mock.patch.object(MultipleTypedModel._meta.db, 'update'),
+                    mock.patch.object(MultipleTypedModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID}
+
+            m._update({
+                'int': 1,
+                'string': 'value',
+                'datetime': datetime.datetime.now(),
+                'list': [],
+                'objectid': ObjectId(),
+            })
+
+            self.assertTrue(update.called)
+
+        # Test a bunch of bad updates.
+        with self.assertRaises(TypeError):
+            m._update({'int': 'value'})
+
+        with self.assertRaises(TypeError):
+            m._update({'datetime': 'value'})
+
+        with self.assertRaises(TypeError):
+            m._update({'list': 'value'})
+
+        with self.assertRaises(TypeError):
+            m._update({'objectid': 'value'})
+
     def test__update_typeerror(self):
         """Test that `_update()` raises `TypeError`."""
 
@@ -618,10 +841,10 @@ class TestDatabase(unittest.TestCase):
         actual = str(e.exception)
         self.assertEqual(actual, expected)
 
+        # TypeError should be raised when required fields are missing.
         m2 = RequiredModel(_id=AN_OBJECT_ID)
         m3 = RequiredModel(_id=AN_OBJECT_ID, a=1, b=2)
 
-        # TypeError should be raised when required fields are missing.
         with self.assertRaises(TypeError) as e:
             m2._update({})
 
@@ -646,6 +869,44 @@ class TestDatabase(unittest.TestCase):
         actual = str(e.exception)
         self.assertEqual(actual, expected)
 
+        # TypeError should be raised when typed fields are of the wrong
+        # type.
+        m4 = TypedModel(_id=AN_OBJECT_ID)
+
+        with self.assertRaises(TypeError) as e:
+            m4._update({'a': 'string'})
+
+        expected = ("The 'TypedModel' object cannot be updated because its 'a'"
+                    " field must be <type 'int'>.")
+        actual = str(e.exception)
+        self.assertEqual(actual, expected)
+
+        with self.assertRaises(TypeError) as e:
+            m4._update({'$set': {'a': 'string'}})
+
+        expected = ("The 'TypedModel' object cannot be updated because its 'a'"
+                    " field must be <type 'int'>.")
+        actual = str(e.exception)
+        self.assertEqual(actual, expected)
+
+        m5 = TypedListModel(_id=AN_OBJECT_ID)
+
+        with self.assertRaises(TypeError) as e:
+            m5._update({'a': ['b']})
+
+        expected = ("The 'TypedListModel' object cannot be updated because its"
+                    " 'a' field must be [<type 'int'>].")
+        actual = str(e.exception)
+        self.assertEqual(actual, expected)
+
+        with self.assertRaises(TypeError) as e:
+            m5._update({'a': [1, 'b']})
+
+        expected = ("The 'TypedListModel' object cannot be updated because its"
+                    " 'a' field must be [<type 'int'>].")
+        actual = str(e.exception)
+        self.assertEqual(actual, expected)
+
     def test__update_use_internal(self):
         """Test the `_update()` method with `use_internal`."""
 
